#include "StdAfx.h"
#include "NListCtrl.h"

CNListCtrl::CNListCtrl(void)
{
	m_pilIPTPresence = NULL;
    m_pilWorkStatus = NULL;
    m_pWorkStatusLayerDlg = NULL;

	m_crNormalText		= COLOR_NORMAL_TEXT;
	m_crNormalBk		= COLOR_NORMAL_BK;
	m_crSelectedText	= COLOR_SELECTED_TEXT;
	m_crSelectedBk		= COLOR_SELECTED_BK;
	m_crLinkNormal		= COLOR_LINK_NORMAL_TEXT;
	m_crLinkHover		= COLOR_LINK_HOVER_TEXT;

	m_nMinColumnWidth = 0;
	m_nImageColumn		= 0;
	m_nIPTPresenceColumn = -1;
    m_nWorkStatusColum = -1;
    m_nPrevIndex = -1;
	m_bShowEmptyText	= FALSE;
	m_nCursor			= CURSOR_DEFAULT;
	m_bCheckBoxHead		= FALSE;

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));       
	g_ftDefault8PT.GetLogFont(&lf);
	m_fontText.CreateFontIndirect(&lf);

	if (m_checkBtnImageList.Create(13, 13, ILC_COLOR32 | ILC_COLOR | ILC_MASK, 2, 2))
	{
		COLORREF crMask = RGB(0, 0, 0);

		CBitmap bmCheck[2];

		LoadBitmapFromPath(bmCheck[0], GetAppImgFile(_T("CheckboxOff.bmp")));
		LoadBitmapFromPath(bmCheck[1], GetAppImgFile(_T("CheckboxOn.bmp")));

		m_checkBtnImageList.Add(&bmCheck[0], crMask);
		m_checkBtnImageList.Add(&bmCheck[1], crMask);

		bmCheck[0].DeleteObject();
		bmCheck[1].DeleteObject();
	}

	m_freezeColList.clear();
	m_linkColList.clear();

}

CNListCtrl::~CNListCtrl(void)
{
	m_fontText.DeleteObject();
	m_checkBtnImageList.DeleteImageList();
	SAFE_DELETE(m_pWorkStatusLayerDlg);
}

BEGIN_MESSAGE_MAP(CNListCtrl, CListCtrl)
	ON_WM_MEASUREITEM_REFLECT()
	ON_WM_PAINT()
	ON_WM_SETCURSOR()
	ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CNListCtrl::OnNMCustomdraw)
	ON_NOTIFY(HDN_BEGINTRACKA, 0, &CNListCtrl::OnHdnBegintrack)
	ON_NOTIFY(HDN_BEGINTRACKW, 0, &CNListCtrl::OnHdnBegintrack)
	ON_NOTIFY_REFLECT(LVN_HOTTRACK, &CNListCtrl::OnLvnHotTrack)
	ON_MESSAGE(LVM_SETCOLUMNWIDTH, &CNListCtrl::OnSetColumnWidth)
	ON_NOTIFY(HDN_DIVIDERDBLCLICKA, 0, &CNListCtrl::OnHdnDividerdblclick)
	ON_NOTIFY(HDN_DIVIDERDBLCLICKW, 0, &CNListCtrl::OnHdnDividerdblclick)
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

BOOL CNListCtrl::AddCheckBoxForHeadCtrl()
{
	if (m_bCheckBoxHead)
		return TRUE;

	CHeaderCtrl* pHeadCtrl = this->GetHeaderCtrl();
	ASSERT(pHeadCtrl->GetSafeHwnd());

	//VERIFY( m_checkHeadCtrl.SubclassWindow(pHeadCtrl->GetSafeHwnd()) );
	//VERIFY( m_checkImgList.Create(IDB_CHECKBOXES, 16, 3, RGB(255,0,255)) );
	//int i = m_checkImgList.GetImageCount();
	pHeadCtrl->SetImageList(&m_checkBtnImageList);

	HDITEM hdItem;
	hdItem.mask = HDI_IMAGE | HDI_FORMAT;
	VERIFY( pHeadCtrl->GetItem(0, &hdItem) );
	hdItem.iImage = 0;
	hdItem.fmt |= HDF_IMAGE;

	VERIFY( pHeadCtrl->SetItem(0, &hdItem) );

	m_bCheckBoxHead = TRUE;

	return TRUE;
}

int CNListCtrl::GetItemImage(int nItem, int nSubItem)
{
	if ((nItem < 0) || nItem >= GetItemCount())
		return FALSE;

	if ((nSubItem < 0) || nSubItem >= GetColumnCount())
		return FALSE;

	LVITEM lvItem;
	lvItem.mask = LVIF_IMAGE;
	lvItem.iItem = nItem;
	lvItem.iSubItem = nSubItem;

	GetItem(&lvItem);

	return lvItem.iImage;
}

BOOL CNListCtrl::SetItemImage(int nItem, int nSubItem, int nImage)
{
	if ((nItem < 0) || nItem >= GetItemCount())
		return FALSE;

	if ((nSubItem < 0) || nSubItem >= GetColumnCount())
		return FALSE;

	LVITEM lvItem;
	lvItem.mask = LVIF_IMAGE;
	lvItem.iItem = nItem;
	lvItem.iSubItem = nSubItem;
	lvItem.iImage = nImage;

	return SetItem(&lvItem);
}

int	CNListCtrl::GetOnlineUserCount(bool bAllUser)
{
	int nRet = 0;

	if (bAllUser)
	{
		int nItemCount = GetItemCount();

		for (int i = 0; i < nItemCount; i++)
		{
			int nImage = GetItemImage(i, 0);

			if (nImage != 0)
				nRet++;
		}
	}
	else
	{
		POSITION pos = GetFirstSelectedItemPosition();

		while (pos)
		{
			int nSelectedItem = GetNextSelectedItem(pos);
			int nImage = GetItemImage(nSelectedItem, 0);

			if (nImage != 0)
				nRet++;
		}
	}

	return nRet;
}

int CNListCtrl::GetColumnCount()
{
	return GetHeaderCtrl()->GetItemCount();
}

void CNListCtrl::MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
	// Listctrl  ̸ ϱ  ߰Ͽ,  ʾ Font ̿  Ͽ
	LOGFONT lf;
	GetFont()->GetLogFont( &lf );
 
	if (lf.lfHeight < 0)
		lpMeasureItemStruct->itemHeight = -lf.lfHeight + 10; 
	else
		lpMeasureItemStruct->itemHeight = lf.lfHeight + 10; 
}

void CNListCtrl::OnNMCustomdraw(NMHDR* pNMHDR, LRESULT* pResult)
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>(pNMHDR);

	int nItem = static_cast<int> (pLVCD->nmcd.dwItemSpec);
	int nSubItem = pLVCD->iSubItem;
	//Trace(_T("(%d,%d) %d,%d,%d,%d\n"), nItem, nSubItem, pLVCD->rcText.left, pLVCD->rcText.right, pLVCD->rcText.top, pLVCD->rcText.bottom);

	*pResult = CDRF_DODEFAULT;

	if (pLVCD->nmcd.dwDrawStage == CDDS_PREPAINT)
	{
		*pResult = CDRF_NOTIFYITEMDRAW;
	}
	else if (pLVCD->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
	{
		*pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
	else if (pLVCD->nmcd.dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
	{
		*pResult = CDRF_SKIPDEFAULT;	// Default Լ  ʰ  ׸

		CDC* pDC = CDC::FromHandle(pLVCD->nmcd.hdc);
		DrawSubItem(nItem, nSubItem, pDC);
	}
}

// bool   CNListCtrl::IsOffline(int iIndex)
// {
// 	LRESULT lResult = ::SendMessage(GetParent()->m_hWnd,UM_GETBUDDY_STATUS,(WPARAM)iIndex,0);
// 	return (lResult == ustOFFLINE)?true:false;
// }

void CNListCtrl::DrawSubItem(int nItem, int nSubItem, CDC *pDC)
{
	CRect rect(0, 0, 0, 0);
	//GetSubItemRect(nItem, nSubItem, LVIR_BOUNDS, rect);
	GetSubItemRect(nItem, nSubItem, LVIR_LABEL, rect);
	if (rect.Width() == 0)
	{
		return;
	}

	/* LVIR_BOUNDS ϸ Ư 쿡 Ʈ  ̰  ׷ ʴ´.
	 * Ƹ SubItem 0  ش Row ü  ͼ  ĥֱ  ׷  .
	 *   ذϱ  LVIR_LABEL  ϸ ̴  , SubItem 0  left  right  ũ ´.
	 * ׷ SubItem 0ΰ left   ִ  óϿ.
	 */
	if (nSubItem == 0)
		rect.left = 0;

	COLORREF crText;
	COLORREF crBkgnd;

	GetDrawColors(nItem, nSubItem, crText, crBkgnd);
	pDC->FillSolidRect(&rect, crBkgnd);

	DWORD dwExStyle  = GetExtendedStyle();

	if ((dwExStyle & LVS_EX_CHECKBOXES) && nSubItem == 0)	// üũڽ
	{
		CPoint ptImg(rect.TopLeft());
		ptImg.Offset(4, 4);

		if (GetCheck(nItem))
			m_checkBtnImageList.Draw(pDC, CHECK_ON, ptImg, ILD_TRANSPARENT);
		else
			m_checkBtnImageList.Draw(pDC, CHECK_OFF, ptImg, ILD_TRANSPARENT);
	}
	else if (nSubItem == m_nImageColumn)	// ޽  
	{
// 		if (IsOffline(nItem) && GET_CONFIG()->EnableFunction(_T("UI_01")))
// 		{
// 			if (m_pilWorkStatus)	// ImageList   츸 
// 			{
// 				int nPresence = 1;
// 
// 				CRect rcImage(0, 0, 0, 0);
// 				GetSubItemRect(nItem, nSubItem, LVIR_ICON, rcImage);
// 
// 				// ׳ ׸    Ǿ, η    ̹   ش.
// 				IMAGEINFO imgInfo;
// 				ZeroMemory(&imgInfo, sizeof(IMAGEINFO));
// 				m_pilWorkStatus->GetImageInfo(nPresence, &imgInfo);
// 				CPoint ptImg(rcImage.TopLeft());
// 				ptImg.Offset(0, (rcImage.Height() - (imgInfo.rcImage.bottom - imgInfo.rcImage.top)) / 2);
// 				m_pilWorkStatus->Draw(pDC, nPresence, ptImg, ILD_TRANSPARENT);
// 			}
// 		}
// 		else
// 		{
			LVITEM rItem;

			//  ε
			ZeroMemory(&rItem, sizeof(LVITEM));
			rItem.mask  = LVIF_IMAGE | LVIF_STATE;
			rItem.iItem = nItem;
			rItem.stateMask = LVIS_SELECTED;
			GetItem(&rItem);

			CRect rcImage(0, 0, 0, 0);
			GetSubItemRect(nItem, nSubItem, LVIR_ICON, rcImage);

			// ׳ ׸    Ǿ, η    ̹   ش.
			IMAGEINFO imgInfo;
			ZeroMemory(&imgInfo, sizeof(IMAGEINFO));
			GetImageList(LVSIL_SMALL)->GetImageInfo(rItem.iImage, &imgInfo);
			CPoint ptImg(rcImage.TopLeft());
			ptImg.Offset(0, (rcImage.Height() - (imgInfo.rcImage.bottom - imgInfo.rcImage.top)) / 2);
			GetImageList(LVSIL_SMALL)->Draw(pDC, rItem.iImage, ptImg, ILD_TRANSPARENT);
/*		}*/

	}
	else if (nSubItem == m_nIPTPresenceColumn)	// IPT   ÷
	{
		// ƮƮѿ IMAGE  ÷ ϹǷ IPT   ش ÷ ؽƮ ְ
		// ش ؽƮ ڷ ȯϿ ̹ ش.
		CString strIPTPresence = GetItemText(nItem, nSubItem);

		if (m_pilIPTPresence && !strIPTPresence.IsEmpty())	// ImageList   츸 
		{
			int nPresence = _ttoi(strIPTPresence);
			CRect rcImage(0, 0, 0, 0);
			GetSubItemRect(nItem, nSubItem, LVIR_ICON, rcImage);

			// ׳ ׸    Ǿ, η    ̹   ش.
			IMAGEINFO imgInfo;
			ZeroMemory(&imgInfo, sizeof(IMAGEINFO));
			m_pilIPTPresence->GetImageInfo(nPresence, &imgInfo);
			CPoint ptImg(rcImage.TopLeft());
			ptImg.Offset(5, (rcImage.Height() - (imgInfo.rcImage.bottom - imgInfo.rcImage.top)) / 2);
			m_pilIPTPresence->Draw(pDC, nPresence, ptImg, ILD_TRANSPARENT);
		}
	}
	else if (nSubItem == m_nWorkStatusColum)	// ٹ  ÷
	{
		// ƮƮѿ IMAGE  ÷ ϹǷαٹ  ش ÷ ؽƮ ְ
		// ش ؽƮ ڷ ȯϿ ̹ ش.
		CString strWorkStatus = GetItemText(nItem, nSubItem);

		if (m_pilWorkStatus && !strWorkStatus.IsEmpty())	// ImageList   츸 
		{
			int nPresence = 1;

			CRect rcImage(0, 0, 0, 0);
			GetSubItemRect(nItem, nSubItem, LVIR_ICON, rcImage);

			// ׳ ׸    Ǿ, η    ̹   ش.
			IMAGEINFO imgInfo;
			ZeroMemory(&imgInfo, sizeof(IMAGEINFO));
			m_pilWorkStatus->GetImageInfo(nPresence, &imgInfo);
			CPoint ptImg(rcImage.TopLeft());
			ptImg.Offset(5, (rcImage.Height() - (imgInfo.rcImage.bottom - imgInfo.rcImage.top)) / 2);
			m_pilWorkStatus->Draw(pDC, nPresence, ptImg, ILD_TRANSPARENT);
		}
	}
	else
	{
		CString strText = GetItemText(nItem, nSubItem);

		if (!strText.IsEmpty())
		{
			CFont *pOldFont = pDC->SelectObject(&m_fontText);

			// get text justification
			HDITEM hditem;
			hditem.mask = HDI_FORMAT;
			GetHeaderCtrl()->GetItem(nSubItem, &hditem);

			int nFmt = hditem.fmt & HDF_JUSTIFYMASK;
			UINT nFormat = DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS | DT_NOPREFIX;

			if (nFmt == HDF_CENTER)
			{
				nFormat |= DT_CENTER;
			}
			else if (nFmt == HDF_LEFT)
			{
				nFormat |= DT_LEFT;
				rect.left += 4;
			}
			else
			{
				nFormat |= DT_RIGHT;
			}

			pDC->SetBkMode(TRANSPARENT);
			pDC->SetTextColor(crText);			
			pDC->SetBkColor(crBkgnd);
			pDC->DrawText(strText, &rect, nFormat);
			pDC->SelectObject(pOldFont);
		}
	}
}

void CNListCtrl::GetDrawColors(int nItem, int nSubItem, COLORREF& colorText, COLORREF& colorBkgnd)
{
	DWORD dwStyle    = GetStyle();
	DWORD dwExStyle  = GetExtendedStyle();

	COLORREF crText  = m_crNormalText;
	COLORREF crBkgnd = m_crNormalBk;

	if (GetItemState(nItem, LVIS_SELECTED))
	{
		if (dwExStyle & LVS_EX_FULLROWSELECT)
		{
			crText  = m_crSelectedText;
			crBkgnd = m_crSelectedBk;

			if (m_hWnd != ::GetFocus())	// Ŀ   õ  
			{
				if (dwStyle & LVS_SHOWSELALWAYS)
				{
					crText  = m_crSelectedText;
					crBkgnd = m_crSelectedBk;
				}
				else
				{
					crText  = m_crSelectedText;
					crBkgnd = m_crSelectedBk;
				}
			}
		}
	}

	if (IsExternalLinkColumn(nSubItem))
	{
		if (m_hoverItem.iItem == nItem && m_hoverItem.iSubItem == nSubItem)
			crText = m_crLinkHover;
		else
			crText = m_crLinkNormal;
	}

	colorText = crText;
	colorBkgnd = crBkgnd;
}

void CNListCtrl::SetRowHeight(int nRowHeight)
{
	CFont font;
	LOGFONT logfont;

	CDC *pDC = GetDC();
	if( !pDC ) return;

	int nRevision=7;

	logfont.lfHeight = -MulDiv(nRowHeight-nRevision, GetDeviceCaps(pDC->m_hDC, LOGPIXELSY), 72);
	ReleaseDC( pDC );

	logfont.lfEscapement	= 0;
	logfont.lfOrientation	= 0;
	logfont.lfWeight		= FW_NORMAL;
	logfont.lfItalic		= FALSE;
	logfont.lfUnderline		= FALSE;
	logfont.lfStrikeOut		= FALSE;
	logfont.lfCharSet		= ANSI_CHARSET;
	logfont.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	logfont.lfQuality		= PROOF_QUALITY;
	logfont.lfPitchAndFamily= DEFAULT_PITCH;
	_tcscpy_s(logfont.lfFaceName, _T("System"));
	font.CreateFontIndirect(&logfont);

	SetFont(&font);

	font.DeleteObject();
}

void CNListCtrl::SetTextFont(CFont* pFont, BOOL bRedraw)
{
	if (pFont == NULL)
		return;

	m_fontText.DeleteObject();

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));       
	pFont->GetLogFont(&lf);
	m_fontText.CreateFontIndirect(&lf);

	if (bRedraw)
		RedrawWindow();
}

void CNListCtrl::SetNormalTextColor(COLORREF crNormalText, COLORREF crNormalBk)
{
	m_crNormalText = crNormalText;
	m_crNormalBk = crNormalBk;
}

void CNListCtrl::SetSelectedTextColor(COLORREF crSelectedText, COLORREF crSelectedBk)
{
	m_crSelectedText = crSelectedText;
	m_crSelectedBk = crSelectedBk;
}

void CNListCtrl::SetImageColumn(int nImageColumn)
{
	ASSERT(nImageColumn >= 0);

	m_nImageColumn = nImageColumn;
}

void CNListCtrl::OnPaint()
{
	Default();

	// Ʈ    ⺻ ޽ 
	if (GetItemCount() <= 0 && m_bShowEmptyText)
	{
		CDC* pDC = GetDC();
		int nSavedDC = pDC->SaveDC();

		CRect rc(0, 0, 0, 0);
		GetWindowRect(&rc);
		ScreenToClient(&rc);
		CHeaderCtrl* pHC = GetHeaderCtrl();
		
		if (pHC != NULL)
		{
			CRect rcH(0, 0, 0, 0);
			pHC->GetItemRect(0, &rcH);
			rc.top += rcH.bottom;
		}

		pDC->SetTextColor(COLOR_NORMAL_TEXT);
		pDC->SetBkMode(TRANSPARENT);
		pDC->FillSolidRect(&rc, COLOR_NORMAL_BK);

		CFont *pOldFont = pDC->SelectObject(&g_ftDefault10PT);
		rc.top += 30;
		pDC->DrawText(LoadStringFromResource(IDS_STRING217), -1, rc, DT_CENTER | DT_SINGLELINE);

		pDC->SelectObject(pOldFont);
		pDC->RestoreDC(nSavedDC);
		ReleaseDC(pDC);
	}
}

void CNListCtrl::ShowEmptyText(BOOL bShow)
{
	m_bShowEmptyText = bShow;
}

BOOL CNListCtrl::SelectAll()
{
	return SetItemState(-1, LVIS_SELECTED, LVIS_SELECTED);
}

void CNListCtrl::SetFreezeColumns(std::list<int> freezeColList)
{
	m_freezeColList = freezeColList;
}

bool CNListCtrl::IsExternalLinkColumn(int nCol)
{
	if (std::find(m_linkColList.begin(), m_linkColList.end(), nCol) != m_linkColList.end())
		return true;

	return false;
}

void CNListCtrl::SetExternalLinkColumns(std::list<int> linkColList)
{
	m_linkColList = linkColList;
}

void CNListCtrl::OnHdnBegintrack(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);

	//  ÷  *pResult = TRUE  Ͽ ¡  ʰ 
	if (std::find(m_freezeColList.begin(), m_freezeColList.end(), phdr->iItem) != m_freezeColList.end())
		*pResult = TRUE;
	else
		*pResult = 0;
}

void CNListCtrl::OnLvnHotTrack(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
	//Trace(_T("OnLvnHotTrack() %d, %d (%d,%d)\n"), pNMLV->iItem, pNMLV->iSubItem, pNMLV->ptAction.x, pNMLV->ptAction.y);

	if (m_hoverItem.iItem != pNMLV->iItem || m_hoverItem.iSubItem != pNMLV->iSubItem)
	{
		CRect rtMouseLeaved(0, 0, 0, 0);
		CRect rtHover(0, 0, 0, 0);

		GetSubItemRect(m_hoverItem.iItem, m_hoverItem.iSubItem, LVIR_BOUNDS, rtMouseLeaved);
		GetSubItemRect(pNMLV->iItem, pNMLV->iSubItem, LVIR_BOUNDS, rtHover);

		m_hoverItem.iItem = pNMLV->iItem;
		m_hoverItem.iSubItem = pNMLV->iSubItem;

		CString strText = GetItemText(pNMLV->iItem, pNMLV->iSubItem);

		if (pNMLV->iItem == -1 || 
			!IsExternalLinkColumn(pNMLV->iSubItem) || 
			strText.IsEmpty())
			m_nCursor = CURSOR_DEFAULT;
		else
			m_nCursor = CURSOR_HAND;

		RedrawWindow(&rtMouseLeaved);
		RedrawWindow(&rtHover);
	}

	*pResult = 0;
}

BOOL CNListCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	switch (m_nCursor)
	{
	case CURSOR_HAND:
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
		return TRUE;
	case CURSOR_DEFAULT:
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
		return TRUE;
	}

	return CListCtrl::OnSetCursor(pWnd, nHitTest, message);
}

void CNListCtrl::SetExternalLinkTextColor(COLORREF crNormal, COLORREF crHover)
{
	m_crLinkNormal = crNormal;
	m_crLinkHover = crHover;
}

void CNListCtrl::ResetHoverStatus()
{
	if (m_hoverItem.iItem != -1)
	{
		CRect rtMouseLeaved(0, 0, 0, 0);

		GetSubItemRect(m_hoverItem.iItem, m_hoverItem.iSubItem, LVIR_BOUNDS, rtMouseLeaved);

		m_hoverItem.iItem = -1;
		m_hoverItem.iSubItem = 0;

		m_nCursor = CURSOR_DEFAULT;

		RedrawWindow(&rtMouseLeaved);
	}
}

void CNListCtrl::SetIPTPresenceImageList(CImageList* pilIPTPresence)
{
	m_pilIPTPresence = pilIPTPresence;
}
void CNListCtrl::SetWorkStatusImageList( CImageList* pilWorkStatus )
{
	m_pilWorkStatus = pilWorkStatus;
}
BOOL CNListCtrl::SetIPTPresenceColumn(int nColumn)
{
	if (m_nImageColumn == nColumn)
		return FALSE;

	m_nIPTPresenceColumn = nColumn;

	return TRUE;
}
BOOL CNListCtrl::SetWorkStatusColumn( int nColumn )
{
	if (m_nImageColumn == nColumn)
		return FALSE;

	m_nWorkStatusColum = nColumn;

	return TRUE;

}


LRESULT CNListCtrl::OnSetColumnWidth(WPARAM wParam, LPARAM lParam)
{
	int nCol = (int)wParam;

	if (std::find(m_freezeColList.begin(), m_freezeColList.end(), nCol) != m_freezeColList.end())
		return FALSE;

	return DefWindowProc(LVM_SETCOLUMNWIDTH, wParam, lParam);
}

void CNListCtrl::OnHdnDividerdblclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);

	*pResult = 0;

	//  ÷   м Ŭص ¡  ʰ 
	if (std::find(m_freezeColList.begin(), m_freezeColList.end(), phdr->iItem) != m_freezeColList.end())
		return;

	Default();
}

void CNListCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	CRect rect(0, 0, 0, 0);
	GetClientRect(&rect);
	CPoint pt(0, 0);
	GetCursorPos(&pt);
	ScreenToClient(&pt);
	UINT uFlags = 0;
	int nIndex = HitTest(pt, &uFlags);
	if (-1 == nIndex || m_nPrevIndex != nIndex)
	{
		m_nPrevIndex = nIndex;
		HideWorkStatusPopup();
	}
	else
	{
		CRect rcWorkStatusBox(0, 0, 0, 0);
		GetWorkStatusBox(nIndex, rcWorkStatusBox);
		if (rcWorkStatusBox.PtInRect(pt))
		{
			if (m_pWorkStatusLayerDlg == NULL)
			{
				m_nPrevIndex = nIndex;
				ShowWorkStatusPopup(nIndex);
			}
		}
	}

	CListCtrl::OnMouseMove(nFlags, point);
}

void CNListCtrl::GetWorkStatusBox( int nCol, CRect& rcBox )
{
	if (m_nWorkStatusColum)
	{
		GetSubItemRect(nCol, m_nWorkStatusColum, LVIR_BOUNDS, rcBox);
	}
	
}

void CNListCtrl::ShowWorkStatusPopup( int nIndex )
{
	CString strWorkStatus = GetItemText(nIndex, m_nWorkStatusColum);
	if (strWorkStatus.IsEmpty())
		return;

	CPoint ptStart(0, 0);

	CRect rcStatusBox(0, 0, 0, 0);
	GetWorkStatusBox(nIndex, rcStatusBox);

	ptStart.x = rcStatusBox.left - 140;
	ptStart.y = rcStatusBox.bottom -10;
	ClientToScreen(&ptStart);


	m_pWorkStatusLayerDlg = new CLayeredDlg();
	m_pWorkStatusLayerDlg->SetImgBGPath(_T("img_bg_layer_02.png"));
	CString strAAA = GetItemText(nIndex,2);
	m_pWorkStatusLayerDlg->SetNoTitleAndMessage(GetItemText(nIndex, m_nWorkStatusColum));
	m_pWorkStatusLayerDlg->Create(CLayeredDlg::IDD, CWnd::GetDesktopWindow());
	m_pWorkStatusLayerDlg->SetWindowPos(&CWnd::wndTop, ptStart.x + 6, ptStart.y + 3, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW);
}

void CNListCtrl::HideWorkStatusPopup()
{
	if (NULL != m_pWorkStatusLayerDlg)
	{
		m_pWorkStatusLayerDlg->DestroyWindow();
		SAFE_DELETE(m_pWorkStatusLayerDlg);
	}
}

void CNListCtrl::SetMinColumnWidth( int nWidth )
{
	m_nMinColumnWidth = nWidth;
}

BOOL CNListCtrl::OnNotify( WPARAM wParam, LPARAM lParam, LRESULT* pResult )
{
	HD_NOTIFY   *pHDN = (HD_NOTIFY*)lParam;

	if((pHDN->hdr.code == HDN_ITEMCHANGINGW || pHDN->hdr.code == HDN_ITEMCHANGINGA) 
		&& pHDN->pitem->cxy < m_nMinColumnWidth)
	{
		*pResult = TRUE;                // disable change
		return TRUE;                    // Processed message
	}

	return CListCtrl::OnNotify(wParam, lParam, pResult);
}
